@@ -5,6 +5,8 @@ Used only for REPL Completion.
 import inspect
 import os
 
+from jedi.parser_utils import get_cached_code_lines
+
 from jedi import settings
 from jedi.evaluate import compiled
 from jedi.cache import underscore_memoization
@@ -13,6 +15,8 @@ from jedi.evaluate.base_context import Context, ContextSet
 from jedi.evaluate.context import ModuleContext
 from jedi.evaluate.cache import evaluator_function_cache
 from jedi.evaluate.compiled.getattr_static import getattr_static
+from jedi.evaluate.compiled.access import compiled_objects_cache
+from jedi.evaluate.compiled.context import create_cached_compiled_object
 
 
 class MixedObject(object):
@@ -37,7 +41,7 @@ class MixedObject(object):
         self.parent_context = parent_context
         self.compiled_object = compiled_object
         self._context = tree_context
-        self.obj = compiled_object.obj
+        self.access_handle = compiled_object.access_handle
 
     # We have to overwrite everything that has to do with trailers, name
     # lookups and filters to make it possible to route name lookups towards
@@ -49,7 +53,7 @@ class MixedObject(object):
         yield MixedObjectFilter(self.evaluator, self)
 
     def __repr__(self):
-        return '<%s: %s>' % (type(self).__name__, repr(self.obj))
+        return '<%s: %s>' % (type(self).__name__, self.access_handle.get_repr())
 
     def __getattr__(self, name):
         return getattr(self._context, name)
@@ -64,7 +68,7 @@ class MixedName(compiled.CompiledName):
         contexts = list(self.infer())
         if not contexts:
             # This means a start_pos that doesn't exist (compiled objects).
-            return (0, 0)
+            return 0, 0
         return contexts[0].name.start_pos
 
     @start_pos.setter
@@ -74,17 +78,11 @@ class MixedName(compiled.CompiledName):
 
     @underscore_memoization
     def infer(self):
-        obj = self.parent_context.obj
-        try:
-            # TODO use logic from compiled.CompiledObjectFilter
-            obj = getattr(obj, self.string_name)
-        except AttributeError:
-            # Happens e.g. in properties of
-            # PyQt4.QtGui.QStyleOptionComboBox.currentText
-            # -> just set it to None
-            obj = None
+        access_handle = self.parent_context.access_handle
+        # TODO use logic from compiled.CompiledObjectFilter
+        access_handle = access_handle.getattr(self.string_name, default=None)
         return ContextSet(
-            _create(self._evaluator, obj, parent_context=self.parent_context)
+            _create(self._evaluator, access_handle, parent_context=self.parent_context)
         )
 
     @property
@@ -105,17 +103,17 @@ class MixedObjectFilter(compiled.CompiledObjectFilter):
 
 
 @evaluator_function_cache()
-def _load_module(evaluator, path, python_object):
-    module = evaluator.grammar.parse(
+def _load_module(evaluator, path):
+    module_node = evaluator.grammar.parse(
         path=path,
         cache=True,
-        diff_cache=True,
+        diff_cache=settings.fast_parser,
         cache_path=settings.cache_directory
     ).get_root_node()
-    python_module = inspect.getmodule(python_object)
-
-    evaluator.modules[python_module.__name__] = module
-    return module
+    # python_module = inspect.getmodule(python_object)
+    # TODO we should actually make something like this possible.
+    #evaluator.modules[python_module.__name__] = module_node
+    return module_node
 
 
 def _get_object_to_check(python_object):
@@ -135,40 +133,43 @@ def _get_object_to_check(python_object):
         raise TypeError  # Prevents computation of `repr` within inspect.
 
 
-def find_syntax_node_name(evaluator, python_object):
+def _find_syntax_node_name(evaluator, access_handle):
+    # TODO accessing this is bad, but it probably doesn't matter that much,
+    # because we're working with interpreteters only here.
+    python_object = access_handle.access._obj
     try:
         python_object = _get_object_to_check(python_object)
         path = inspect.getsourcefile(python_object)
     except TypeError:
         # The type might not be known (e.g. class_with_dict.__weakref__)
-        return None, None
+        return None
     if path is None or not os.path.exists(path):
         # The path might not exist or be e.g. <stdin>.
-        return None, None
+        return None
 
-    module = _load_module(evaluator, path, python_object)
+    module_node = _load_module(evaluator, path)
 
     if inspect.ismodule(python_object):
         # We don't need to check names for modules, because there's not really
         # a way to write a module in a module in Python (and also __name__ can
         # be something like ``email.utils``).
-        return module, path
+        code_lines = get_cached_code_lines(evaluator.grammar, path)
+        return module_node, module_node, path, code_lines
 
     try:
         name_str = python_object.__name__
     except AttributeError:
         # Stuff like python_function.__code__.
-        return None, None
+        return None
 
     if name_str == '<lambda>':
-        return None, None  # It's too hard to find lambdas.
+        return None  # It's too hard to find lambdas.
 
     # Doesn't always work (e.g. os.stat_result)
-    try:
-        names = module.get_used_names()[name_str]
-    except KeyError:
-        return None, None
+    names = module_node.get_used_names().get(name_str, [])
     names = [n for n in names if n.is_definition()]
+    if not names:
+        return None
 
     try:
         code = python_object.__code__
@@ -184,33 +185,40 @@ def find_syntax_node_name(evaluator, python_object):
         # There's a chance that the object is not available anymore, because
         # the code has changed in the background.
         if line_names:
-            return line_names[-1].parent, path
+            names = line_names
 
+    code_lines = get_cached_code_lines(evaluator.grammar, path)
     # It's really hard to actually get the right definition, here as a last
     # resort we just return the last one. This chance might lead to odd
     # completions at some points but will lead to mostly correct type
     # inference, because people tend to define a public name in a module only
     # once.
-    return names[-1].parent, path
+    return module_node, names[-1].parent, path, code_lines
 
 
-@compiled.compiled_objects_cache('mixed_cache')
-def _create(evaluator, obj, parent_context=None, *args):
-    tree_node, path = find_syntax_node_name(evaluator, obj)
+@compiled_objects_cache('mixed_cache')
+def _create(evaluator, access_handle, parent_context, *args):
+    compiled_object = create_cached_compiled_object(
+        evaluator, access_handle, parent_context=parent_context.compiled_object)
 
-    compiled_object = compiled.create(
-        evaluator, obj, parent_context=parent_context.compiled_object)
-    if tree_node is None:
+    result = _find_syntax_node_name(evaluator, access_handle)
+    if result is None:
         return compiled_object
 
-    module_node = tree_node.get_root_node()
+    module_node, tree_node, path, code_lines = result
+
     if parent_context.tree_node.get_root_node() == module_node:
         module_context = parent_context.get_root_context()
     else:
-        module_context = ModuleContext(evaluator, module_node, path=path)
+        module_context = ModuleContext(
+            evaluator, module_node,
+            path=path,
+            code_lines=code_lines,
+        )
         # TODO this __name__ is probably wrong.
         name = compiled_object.get_root_context().py__name__()
-        imports.add_module(evaluator, name, module_context)
+        if name is not None:
+            imports.add_module_to_cache(evaluator, name, module_context)
 
     tree_context = module_context.create_context(
         tree_node,
@@ -218,7 +226,7 @@ def _create(evaluator, obj, parent_context=None, *args):
         node_is_object=True
     )
     if tree_node.type == 'classdef':
-        if not inspect.isclass(obj):
+        if not access_handle.is_class():
             # Is an instance, not a class.
             tree_context, = tree_context.execute_evaluated()
 
@@ -228,4 +236,3 @@ def _create(evaluator, obj, parent_context=None, *args):
         compiled_object,
         tree_context=tree_context
     )
-
